#ifndef GROUPS_H
#define GROUPS_H
/*****************************
This file has classes for defining and manipulating groups.
The allowed space rotation groups are restricted to subgroups of the tetragonal group D_4h 
(Schoenflies notation) with 16 elements.
It might be generalizable to the 48-element octahedral group O_h, 
the symmetry group of a cube, which also has 3-fold rotations.
A member g of such a group is represented by 4 bits (from least to most 
significant) gx,gy,gz,gp (=0 or 1)where g = mv^gp mz^gz my^gy mx^gx
where mx is a mirror in the yz plane (x<->-x), etc
and mv is a mirror in a plane containing the z axis, at 45 degrees 
from the coordinate planes (x <-> y).
Note that mx, my, and mz all commute, but mv does NOT commute with mx or my:
mv*my=mx*mv, not my*mv.
Thus the integer representing g is ig=gx+2*gy+4*gz+8*gp.
We can write g by giving the binary representation of ig: 1000=mv, 
1001=mv*mx, which takes x,y,z ->-x,y,z -> y,-x,z.
We also use spin-space rotations taken from the direct-product group
D_4h(space) x D_4h(spin), but to preserve energy or dynamics the spin and space rotations
must be the same, except for inversion.So instead of a 16x16=256 element group of all 
possible products g^space g^spin of space and spin rotations, we have only 16x2=32.  
And we can only use each space rotation once, so max size group is 16.
Represent group by giving generators (each with a spin-inversion bit in addition to 
rotation iRot).
*******************************/

// #include "Systm.h" NO! is highest level

//#include "Lowest.h" //is in system?

// #include "utility.h"
#include <string>
#include <list>
// using namespace std;
class Config;
class MVector;

#define NumRots 16 
#define MaxGens 4
// These functions are not in a class:
MVector RotV(RotType R, MVector V);

inline RotType CommutePerm(RotType R, RotType Perm)
{  if(Perm&8) return ((R&1)<<1) | ((R&2)>>1) | (R&28); // 28 has all bits but 0&1
	  else return R; // this could be generalized to 6 permutations with switch
}

inline RotType Mult(RotType RA, RotType RB)// multiply two rotations (R=RA*RB)
   {return CommutePerm(RA,RB)^RB;} // can be 4-bit space or include spin-inv bit

inline RotType Inverse(RotType R) {return CommutePerm(R,R);}

// horrible trouble initializing static arrays. Try making it global:
const string RotNames[16]={"1","mx","my","R180",  "mz","Ry","Rx","i",
  "m/","R270","R90","m\\",  "R/","iR90","iR270","R\\"};

class Nesting // a nesting of groups, determined by generators read in
{  public:    //indexed by iGroup=0(identity group),i=1(1 generator), etc. 
   string NesName;
   // what for?: list<Nesting> DUMMY;
   RotType Gen[MaxGens]; // only NumGens actually, <=4
  // these could be static:
   RotType GRots[NumRots]; // group elements, ordered consistently with nesting 
     // note that these are space-spin rotations, with spin inversion bit packed
   RotType SizeSub[MaxGens+1]; // group labels ident=0,1,2,3,4=D4h
   //RotType *Coset; //RotType SizeCoset;
   //bool InvertSpin[MaxGens]; // whether extra inversion on spin
   // could in principle have 2 nestings differing only in InvertSpin, but unlikely.
   RotType genOrder[MaxGens]; // order of the generator (2 or 4)
   bool Preserved[MaxGens+1]; // whether group is preserved by LL equation
   RotType NumGens; // number of generators (<=4) 
   // RotType NumGensSub; //# generators actually input:subgroup
   //unsigned char GpOrder; // or does this follow from NumGensSub?
   static CellIDType NWedge; // # cells in 1/16 wedge
   static CellIDType** RotCell;
   Nesting(){}; // constructor
   ~Nesting(){}; //unused? destructor
   void ReadNesting(ifstream& Fil); // reads info from script file
   void MakeWedge();
   Config Project(Config,RotType iGroup);
   // later: Config Breaking(Config, RotType iGroup); // breaking from iGroup+1 to iGroup
};

class NestHolder // there will be only one of these
{  // is it really needed? or should Symmetries be in Systm?
 public:
	 // general parameters for all nests
	 list<Nesting> Nestings;
	 list<Nesting>::iterator ProjToNes; short ProjToGp; // group for evolve to project to
  	NestHolder(); // called the first time a group is defined
	list<Nesting>::iterator NesNamed(string Nam);
};

class Symmetry // a representation of the continuous rotation group C_infinity_h
	// indexed by l_ and parity
{  //private
   real Cos, Sin;
   CellIDType** NbrID;  real** NbrW; // weight of nbr
   void RotC(Config& M0,Config& MRot);// Rotate Configuration
 public:
   Symmetry(short,bool); // constructor
   ~Symmetry();
   //string SymName;
   short l_; // azimuthal ang. mom. quantum number
   bool odd; // parity: true if odd
   enum usages {ProjTo,ProjOut,Ignore} usage;
   void Project_l(Config&,Config&);
   void ProjectOut(Config&,Config&);
   friend ostream& operator<< (ostream&, Symmetry) ;
};  // end class Symmetry

class SymHolder // there will be only one of these
{  // is it really needed? or should Nestings be in Systm?
 public:
	 // general parameters for all nests
	 list<Symmetry> Symmetries;
  	SymHolder(); // called the first time a Symmetry is defined
	list<Symmetry>::iterator FindSym(short l_1,bool odd1);
};

Config MakeSeed(short w,bool odd, short j);

#endif |GROUPS_H